# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './kuafu/resources/ui_library.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_librarywidget(object):
    def setupUi(self, librarywidget):
        librarywidget.setObjectName("librarywidget")
        librarywidget.resize(970, 701)
        self.horizontalLayout = QtWidgets.QHBoxLayout(librarywidget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter = QtWidgets.QSplitter(librarywidget)
        self.splitter.setMinimumSize(QtCore.QSize(0, 0))
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.tabWidget_2 = QtWidgets.QTabWidget(self.splitter)
        self.tabWidget_2.setMinimumSize(QtCore.QSize(200, 0))
        self.tabWidget_2.setMaximumSize(QtCore.QSize(600, 16777215))
        self.tabWidget_2.setObjectName("tabWidget_2")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontal_Layout = QtWidgets.QHBoxLayout(self.tab_3)
        self.horizontal_Layout.setContentsMargins(0, 0, 0, 0)
        self.horizontal_Layout.setObjectName("horizontal_Layout")
        self.splitter_1 = QtWidgets.QSplitter(self.tab_3)
        self.splitter_1.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.splitter_1.setOrientation(QtCore.Qt.Vertical)
        self.splitter_1.setObjectName("splitter_1")
        self.fileview = QtWidgets.QTreeView(self.splitter_1)
        self.fileview.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.fileview.setObjectName("fileview")
        self.annotview = QtWidgets.QTreeView(self.splitter_1)
        self.annotview.setObjectName("annotview")
        self.horizontal_Layout.addWidget(self.splitter_1)
        self.tabWidget_2.addTab(self.tab_3, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.tabWidget_2.addTab(self.tab_4, "")
        self.splitter_doc = QtWidgets.QSplitter(self.splitter)
        self.splitter_doc.setMinimumSize(QtCore.QSize(600, 0))
        self.splitter_doc.setOrientation(QtCore.Qt.Vertical)
        self.splitter_doc.setObjectName("splitter_doc")
        self.preview_graphicsview = DocGraphicsView(self.splitter_doc)
        self.preview_graphicsview.setObjectName("preview_graphicsview")
        self.preview_graphicsview_2 = DocGraphicsView(self.splitter_doc)
        self.preview_graphicsview_2.setObjectName("preview_graphicsview_2")
        self.thumb_graphicsview = ThumbGraphicsView(self.splitter)
        self.thumb_graphicsview.setMinimumSize(QtCore.QSize(150, 0))
        self.thumb_graphicsview.setMaximumSize(QtCore.QSize(600, 16777215))
        self.thumb_graphicsview.setObjectName("thumb_graphicsview")
        self.horizontalLayout.addWidget(self.splitter)

        self.retranslateUi(librarywidget)
        self.tabWidget_2.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(librarywidget)

    def retranslateUi(self, librarywidget):
        _translate = QtCore.QCoreApplication.translate
        librarywidget.setWindowTitle(_translate("librarywidget", "Form"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_3), _translate("librarywidget", "Tab 1"))
        self.tabWidget_2.setTabText(self.tabWidget_2.indexOf(self.tab_4), _translate("librarywidget", "Tab 2"))
from docgraphicsview import DocGraphicsView
from thumbgraphicsview import ThumbGraphicsView
