# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './kuafu/resources/ui_annotation_item.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_annotationItemWidget(object):
    def setupUi(self, annotationItemWidget):
        annotationItemWidget.setObjectName("annotationItemWidget")
        annotationItemWidget.resize(451, 363)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(annotationItemWidget.sizePolicy().hasHeightForWidth())
        annotationItemWidget.setSizePolicy(sizePolicy)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(annotationItemWidget)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_indicator = QtWidgets.QLabel(annotationItemWidget)
        self.label_indicator.setMinimumSize(QtCore.QSize(5, 0))
        self.label_indicator.setMaximumSize(QtCore.QSize(5, 16777215))
        self.label_indicator.setObjectName("label_indicator")
        self.horizontalLayout_3.addWidget(self.label_indicator)
        self.groupBox = QtWidgets.QGroupBox(annotationItemWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setTitle("")
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_date = QtWidgets.QLabel(self.groupBox)
        self.label_date.setMaximumSize(QtCore.QSize(16777215, 20))
        self.label_date.setObjectName("label_date")
        self.horizontalLayout.addWidget(self.label_date)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label_author = QtWidgets.QLabel(self.groupBox)
        self.label_author.setMaximumSize(QtCore.QSize(16777215, 20))
        self.label_author.setObjectName("label_author")
        self.horizontalLayout.addWidget(self.label_author)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.label_title = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_title.sizePolicy().hasHeightForWidth())
        self.label_title.setSizePolicy(sizePolicy)
        self.label_title.setObjectName("label_title")
        self.verticalLayout.addWidget(self.label_title)
        self.edit_comments = QtWidgets.QLineEdit(self.groupBox)
        self.edit_comments.setObjectName("edit_comments")
        self.verticalLayout.addWidget(self.edit_comments)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.pushButton_more = QtWidgets.QPushButton(self.groupBox)
        self.pushButton_more.setMaximumSize(QtCore.QSize(30, 30))
        self.pushButton_more.setObjectName("pushButton_more")
        self.horizontalLayout_2.addWidget(self.pushButton_more)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3.addWidget(self.groupBox)

        self.retranslateUi(annotationItemWidget)
        QtCore.QMetaObject.connectSlotsByName(annotationItemWidget)

    def retranslateUi(self, annotationItemWidget):
        _translate = QtCore.QCoreApplication.translate
        annotationItemWidget.setWindowTitle(_translate("annotationItemWidget", "Form"))
        self.label_indicator.setText(_translate("annotationItemWidget", "TextLabel"))
        self.label_date.setText(_translate("annotationItemWidget", "TextLabel"))
        self.label_author.setText(_translate("annotationItemWidget", "TextLabel"))
        self.label_title.setText(_translate("annotationItemWidget", "TextLabel"))
        self.pushButton_more.setText(_translate("annotationItemWidget", "..."))
