# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './kuafu/resources/ui_document.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_doc_view(object):
    def setupUi(self, doc_view):
        doc_view.setObjectName("doc_view")
        doc_view.resize(843, 432)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(doc_view.sizePolicy().hasHeightForWidth())
        doc_view.setSizePolicy(sizePolicy)
        self.horizontalLayout = QtWidgets.QHBoxLayout(doc_view)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.splitter_main = QtWidgets.QSplitter(doc_view)
        self.splitter_main.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_main.setObjectName("splitter_main")
        self.tabWidget1 = QtWidgets.QTabWidget(self.splitter_main)
        self.tabWidget1.setMaximumSize(QtCore.QSize(400, 16777215))
        self.tabWidget1.setObjectName("tabWidget1")
        self.outline = QtWidgets.QWidget()
        self.outline.setObjectName("outline")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.outline)
        self.horizontalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.treeView_outline = QtWidgets.QTreeView(self.outline)
        self.treeView_outline.setObjectName("treeView_outline")
        self.horizontalLayout_5.addWidget(self.treeView_outline)
        self.tabWidget1.addTab(self.outline, "")
        self.thumbs = QtWidgets.QWidget()
        self.thumbs.setObjectName("thumbs")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.thumbs)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.graphicsView_thumbs = DocGraphicsView(self.thumbs)
        self.graphicsView_thumbs.setObjectName("graphicsView_thumbs")
        self.horizontalLayout_6.addWidget(self.graphicsView_thumbs)
        self.tabWidget1.addTab(self.thumbs, "")
        self.splitter_doc = QtWidgets.QSplitter(self.splitter_main)
        self.splitter_doc.setOrientation(QtCore.Qt.Vertical)
        self.splitter_doc.setObjectName("splitter_doc")
        self.graphicsView_doc1 = DocGraphicsView(self.splitter_doc)
        self.graphicsView_doc1.setObjectName("graphicsView_doc1")
        self.graphicsView_doc2 = DocGraphicsView(self.splitter_doc)
        self.graphicsView_doc2.setObjectName("graphicsView_doc2")
        self.tabWidget2 = QtWidgets.QTabWidget(self.splitter_main)
        self.tabWidget2.setMinimumSize(QtCore.QSize(0, 0))
        self.tabWidget2.setMaximumSize(QtCore.QSize(400, 16777215))
        self.tabWidget2.setTabBarAutoHide(True)
        self.tabWidget2.setObjectName("tabWidget2")
        self.annotations = QtWidgets.QWidget()
        self.annotations.setObjectName("annotations")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.annotations)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.annotScrollArea = CustomScrollArea(self.annotations)
        self.annotScrollArea.setWidgetResizable(True)
        self.annotScrollArea.setObjectName("annotScrollArea")
        self.annotScrollAreaContents = QtWidgets.QWidget()
        self.annotScrollAreaContents.setGeometry(QtCore.QRect(0, 0, 122, 428))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.annotScrollAreaContents.sizePolicy().hasHeightForWidth())
        self.annotScrollAreaContents.setSizePolicy(sizePolicy)
        self.annotScrollAreaContents.setObjectName("annotScrollAreaContents")
        self.annotScrollAreaContents_Layout = QtWidgets.QHBoxLayout(self.annotScrollAreaContents)
        self.annotScrollAreaContents_Layout.setContentsMargins(0, 0, 0, 0)
        self.annotScrollAreaContents_Layout.setObjectName("annotScrollAreaContents_Layout")
        self.annotScrollArea.setWidget(self.annotScrollAreaContents)
        self.verticalLayout_4.addWidget(self.annotScrollArea)
        self.tabWidget2.addTab(self.annotations, "")
        self.horizontalLayout.addWidget(self.splitter_main)

        self.retranslateUi(doc_view)
        self.tabWidget1.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(doc_view)

    def retranslateUi(self, doc_view):
        _translate = QtCore.QCoreApplication.translate
        doc_view.setWindowTitle(_translate("doc_view", "Form"))
        self.tabWidget1.setTabText(self.tabWidget1.indexOf(self.outline), _translate("doc_view", "Outline"))
        self.tabWidget1.setTabText(self.tabWidget1.indexOf(self.thumbs), _translate("doc_view", "Thumbs"))
        self.tabWidget2.setTabText(self.tabWidget2.indexOf(self.annotations), _translate("doc_view", "Annotations"))
from customscrollarea import CustomScrollArea
from docgraphicsview import DocGraphicsView
